/***
 * OW2 FraSCAti Tinfi
 * Copyright (C) 2007-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.ow2.frascati.tinfi.control.content;

import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.julia.control.lifecycle.ChainedIllegalLifeCycleException;
import org.objectweb.fractal.julia.control.lifecycle.LifeCycleCoordinator;

/**
 * Mixin layer to be included in the lifecycle controller for managing the
 * EagerInit and EagerDestroy policy.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 */
public abstract class LifeCycleEagerMixin implements LifeCycleCoordinator {
    
    // -------------------------------------------------------------------------
    // Private constructor
    // -------------------------------------------------------------------------

    private LifeCycleEagerMixin() {}

    // -------------------------------------------------------------------------
    // Implementation of the LifeCycleCoordinator interface
    // -------------------------------------------------------------------------

    public boolean setFcStarted() throws IllegalLifeCycleException {
        
        /*
         * Notify the SCA content controller that the component has been
         * started. This gives the opportunity to eager initialize the component
         * if the @EagerInit annotation has been set.
         */
        try {
            _this_weaveableSCACC.eagerInit();
        }
        catch( ContentInstantiationException ie ) {
            throw new ChainedIllegalLifeCycleException(ie,null,"");
        }

        return _super_setFcStarted();
    }
    
    /**
     * @since 1.2.1
     */
    public boolean setFcStopped() throws IllegalLifeCycleException {
        
        /*
         * Notify the SCA content controller that the component has been
         * stopped. This gives the opportunity to eager destory the component
         * if the @EagerDestroy annotation has been set.
         */
        try {
            _this_weaveableSCACC.eagerDestroy();
        }
        catch( ContentInstantiationException ie ) {
            throw new ChainedIllegalLifeCycleException(ie,null,"");
        }

        return _super_setFcStopped();
    }
    

    // -------------------------------------------------------------------------
    // Fields and methods required by the mixin class in the base class
    // -------------------------------------------------------------------------

    public abstract boolean _super_setFcStarted() throws IllegalLifeCycleException;
    public abstract boolean _super_setFcStopped() throws IllegalLifeCycleException;

    /**
     * The {@link SCAContentController} interface of the component to which
     * this controller object belongs.
     */
    public SCAContentController _this_weaveableSCACC;
}
